
/************************************************************************/
/* Kiwanda                                                            */
/* Algemene definities voor operatie                                    */
/* Alle software in dit bestand is                                      */
/* (c) 2001,2002,2003 Kiwanda Embedded Systemen                              */
/* $Id: algdef.h,v 1.18 2006/03/31 08:44:02 oldenburgh Exp $                                                                 */
/************************************************************************/

#ifndef KIWANDA_ALGDEF_H
#define KIWANDA_ALGDEF_H


#include <stdio.h>

/* standaard definities */

#ifndef PI
#define PI  3.141592654
#endif

#ifdef  FALSE
#define False		FALSE
#else
#define False		0
#endif

#ifdef  TRUE
#define True		TRUE
#else
#define True		1
#endif

#ifndef NULL
#define NULL            0
#endif

#ifndef Null
#define Null            NULL
#endif

#define forever         while(1==1)

/* Basic Types
 */
typedef char	 		Char;
typedef unsigned char		UChar;
typedef unsigned int		UInt;
typedef int			Int;
typedef float			Float;
typedef double			Double;

/* Sized Types
 */
typedef signed char		Int8;
typedef unsigned char		UInt8;
typedef signed short		Int16;
typedef unsigned short		UInt16;
typedef signed long 		Int32;
typedef unsigned long		UInt32;
typedef long long		Int64;  /* wellicht niet nodig voor een 8 bit processor !*/
typedef unsigned long long	UInt64; 

/* Bool Type  */
typedef UInt32			Bool32;	
typedef UInt16			Bool16;
typedef UInt8			Bool8;

#define KiwandaAdres            void *

typedef Float       NatuurkundeFloatType;
typedef Float       WiskundeFloatType;


/* algemene funktie foutmelding */

typedef enum
{
    Ok=0,
    Waarschuwing,
    Fout,
    Ernstig,
    Rampzalig,
    ReserveFoutCode
} FoutCode;

typedef enum
{
    SchakelaarUit=0,
    SchakelaarAan,
    SchakelaarEinde
} Schakelaar;

#ifdef AVRTARGET
typedef UInt8 Teller;           /* standaard breedte van de AVR */ 
#endif

#ifdef ARMTARGET
typedef UInt32 Teller;          /* standaard breedte van de ARM7 */ 
#endif

#ifdef KIWANDA_INTEL386
typedef UInt32 Teller;          /* standaard breedte van de Intel 386 */ 
#endif


typedef UInt16 KiwandaNanoSec;
typedef UInt32 KiwandaMicroSec;
/* Tijd voor de ms wordt gemeten in 16 bits */
/* bereik is dan (2^16-1) / 1000 = 65 seconden */
/* schaal van 1 ms ... 1 minuut */
typedef UInt16 KiwandaMilliSec;

typedef UInt8 KiwandaSeconde;
typedef UInt8 KiwandaMinuut;
typedef UInt8 KiwandaUur;    

typedef enum
{
    Maandag=0,
    Dinsdag,
    Woensdag,
    Donderdag,
    Vrijdag,
    Zaterdag,
    Zondag
} KiwandaDag;

typedef enum
{
    Januari,
    Februari,
    Maart,
    April,
    Mei,
    Juni,
    Juli,
    Augustus,
    September,
    Oktober,
    November,
    December
} KiwandaMaand;

typedef UInt8 KiwandaJaar;   /* van 2000 tot 2255 */

/* funkties en inline funkties */

typedef void (*KiwandaFunktie) () ;  /* funktieprototype voor interrupt funktie */



/* bitsgewijze operatie macros . Geef bits aan als nummer en niet als 2 tot de macht (nummer) */ 
#define SetBit(x,y)          x |= (1<<y) /* Set bit y in byte x*/ 
#define ClearBit(x,y)        x &= (~(1<<(y))) /* Clear bit y in byte x*/ 
#define CheckBit(x,y)        (x & (1<<y))  /* Check bit y in byte x*/
#define EnterBit(x,y,b)      if (b)  SetBit(x,y) else  ClearBit(x,y)  /* zet bit y in byte x gelijk aan b */
#define CopyBit(a,b,x,y)     EnterBit((x),(y), (CheckBit(a,b)) ) /* kopieer bit b in byte a naar bit y in byte x */  //    (x =  (x&(~(1<<(y))))  | ((a & (1<<b) ) << (y-b) ) )  /* kopieer bit b in byte a naar bit y in byte x */
#define ToggleBit(x,y)       x = (x ^ (1<<y));  /* verander bit y in byte x */

/* bitsgewijze operatie macros . Geef bits als 2 tot de macht (nummer) */ 
/* Werkt in samenhang met Atmel AT91 bitdefinities */
#define SetBitM(var,bit)          var |= (bit) /* Set bit bit in byte var*/ 
#define ClearBitM(var,bit)        var &= (~(bit))  /* Clear bit y in bbitte var*/ 
#define CheckBitM(var,bit)        (var & (bit))  /* Check bit y in byte var*/
#define EnterBitM(var,bit,b)      if (b) SetBitM(var,bit) else ClearBitM(var,bit)  /* zet bit bit in byte var gelijk aan b */
#define CoybitBitM(src,srcbit,dest,destbit)     EnterBitM((dest),(destbit), (CheckBitM(src,srcbit)) ) /* kopieer bit b in byte a naar bit bit in byte var */  //    (var =  (var&(~(1<<(bit))))  | ((a & (1<<b) ) << (bit-b) ) )  /* kopieer bit b in bbitte a naar bit bit in byte var */
#define ToggleBitM(var,bit)       var = (var ^ (bit))  /* flip bit y in byte var */





/* tel b bij a op binnen 'bits' aantal bits. Bij een overflow over 'bits' bits wordt slechts het restant doorgegeven */
/* Werkt alleen bij unsigned integers !!! */
#define kiwandaTelBinnenBits(a,b, bits)  ( ( (a+b) < (1<<bits) ) ? a+b : (a+b-1-(1<<bits)))  

    
/* Bit definities */
#define KBit0                0x01
#define KBit1                0x02
#define KBit2                0x04
#define KBit3                0x08
#define KBit4                0x10
#define KBit5                0x20
#define KBit6                0x40
#define KBit7                0x80


/* Serieele hulpjes */
#define HoogWoord(woord)    ((UInt16) 0xffff & (woord >> 16))  /* In een 32 bit geheugen, is dit het Most Significant Word (16 bit) */
#define LaagWoord(woord)    ((UInt16) 0xffff & woord)          /* In een 32 bit geheugen, is dit het Least Significant Word (16 bit) */
#define HoogByte(woord)    ((UInt8) 0xff & (woord >> 8))       /* In een 16 bit geheugen, is dit het Most Significant Byte (8 bit) */
#define LaagByte(woord)    ((UInt8) 0xff & woord)              /* In een 16 bit geheugen, is dit het Least Significant Byte (8 bit) */

#define Hoog              1
#define Laag              0
                              
#define ReturnIsOk(a)  ( ( (a) == Ok ) ? True : False )
#define ReturnIsNietOk(a)  ( ( (a) != Ok ) ? True : False )

#endif  /* ALGDEF_H */





