
/************************************************************************
grafische funkties API. Hier staat wxWidget code om grafieken in
assenstelsels te tekenen.

(c) 2004-2005 Kiwanda Embedded Systemen
$Id: grafiek.cpp,v 1.1.1.1 2006/02/22 18:17:27 oldenburgh Exp $
************************************************************************/


#include "grafiek.h"

#include <wx/log.h>
#include <wx/pen.h>
#include <wx/dcclient.h>
#include <wx/msgdlg.h>

BEGIN_EVENT_TABLE( EenDimensieGrafiek, wxPanel)

    EVT_MOTION(EenDimensieGrafiek::toonGrafiekWaarde)
    EVT_PAINT(EenDimensieGrafiek::teken)

END_EVENT_TABLE();


EenDimensieGrafiek::EenDimensieGrafiek(wxWindow* parent, 
				       int id,
				       const wxPoint &pos, 
				       const wxSize &size, 
				       long style):
    wxPanel(parent, id, pos, size, wxTAB_TRAVERSAL)
{

    /* data labels */
    tekenDataLabels = True;
    getoondeDataXCoordinaat = 0;

    dataGeladen = attrGezet = assenstelselGezet= False;

    afbeeldingsStart = 0;

    schaal = 1;

    data = (wxRealPoint *) Null;

    toevoegPunt = 0;

}


EenDimensieGrafiek::~EenDimensieGrafiek()
{

    if (data != (wxRealPoint *)Null)
	delete [] data;

}


FoutCode EenDimensieGrafiek::vraagDataRuimte(const UInt16 aantal)
{

  wxASSERT_MSG( aantal > 0, wxT("(vraagDataRuimte) aantal kleiner of gelijk aan 0"));

  dataGrootte = aantal;
  data = new wxRealPoint[aantal];

  return(Ok);

}

FoutCode EenDimensieGrafiek::zetAlleData(const wxRealPoint *waarden)        /* zet alle data ineens */
{

  wxASSERT_MSG( waarden != (wxRealPoint *)Null,
		wxT("(zetAlleData) Null pointer aangeboden"));


//  wxLogDebug(wxT("(zetAlleData) Data geladen."));

  memcpy(data,waarden,dataGrootte*sizeof(wxRealPoint));

  dataGeladen = True;

  Refresh();

  return(Ok);

}

FoutCode EenDimensieGrafiek::zetDataPunt(UInt16 nummer,
					 const wxRealPoint waarde)
{

  wxASSERT_MSG( nummer < dataGrootte,
		wxT("(zetDataPunt) te grote index aangeboden"));

  *(data +nummer) = waarde;

  Refresh();

  return(Ok);


}

FoutCode EenDimensieGrafiek::voegDataPuntToe(const wxRealPoint waarde)     /* voegt een punt toe op de volgende plek */
{
    Teller i;

    if (++toevoegPunt == dataGrootte)
    {
	toevoegPunt =0;
	for (i=0;i<dataGrootte;i++)
	{
	    (data+i)->x = 0.0;
	    (data+i)->y = 0.0;
	}
    }

    *(data+toevoegPunt) = waarde;

    Refresh();

    return(Ok);
}

FoutCode EenDimensieGrafiek::zetAssenstelsel(wxSize negatiefBereik,  
					     wxSize positiefBereik,
					     const wxSize iAfstand,       /* aantal ticks tussen labels */
					     const Bool8 autoYSchaal,     /* bepaal zelf min of max */
					     UInt16 *secties )      /* return: aantal secties in de dataset die afgebeeld kunnen worden in het assenstelsel */
{
  Teller i;

  Int16 waarde,negBereik=0,posBereik=0;

  wxASSERT_MSG((dataGeladen == True), 
	       wxT("Kan niet assenstelsel zetten zonder geladen data."));


  grafiekGrootte.x = positiefBereik.GetWidth() - negatiefBereik.GetWidth();

  if (autoYSchaal == True)
    {

      for (i=0;i<dataGrootte;i++)
	{
	  waarde = (Int16) (data+i)->y;
	  if ( (waarde < 0) && (  (data+i)->y > 0))  /* overflow */
	    {
	      wxLogDebug(wxT("waarde = %d, data->y = %f"),waarde,  (data+i)->y);
		  wxMessageDialog dialoog( this,
					   wxT("Your dataset for this graph exceeds data limits.\nPlease adjust the scale format."),
					   wxT("Error configuring data"),
					   wxOK | wxICON_ERROR );
		  dialoog.ShowModal();
		  return(Waarschuwing);
	    }

	  if (waarde > posBereik)
	    posBereik = waarde;
	  if (waarde < negBereik)
	    negBereik = waarde;
	}
      positiefBereik.y = posBereik+1;   /* 10% marge erbij */
      negatiefBereik.y = negBereik-1;
    }


  grafiekGrootte.y = positiefBereik.GetHeight() - negatiefBereik.GetHeight();
  negatieveLimiet =  negatiefBereik;

  grafiekBasis.x = negatiefBereik.GetWidth();
  grafiekBasis.y = negatiefBereik.GetHeight();
 
  /* Bepaal de breedte van een interval*/
  intervalAfstand =  iAfstand;

  if (secties != (UInt16 *)Null)
  {
      *secties = dataGrootte/grafiekGrootte.x;

      if ((dataGrootte % grafiekGrootte.x) != 0)
	  *(secties) ++;
  }

  assenstelselGezet = True;

  Refresh();

  return(Ok);
}

FoutCode EenDimensieGrafiek::zetAttributen(wxColour asKleur,
					   wxBrush asKwast,
					   wxColour grafKleur,
					   wxBrush grafKwast,
					   wxString xLabel,
					   wxString yLabel )

{

  assenstelselKleur = asKleur;
  assenstelselBlokKwast = asKwast;

  grafiekKleur = grafKleur;
  grafiekBlokKwast = grafKwast;

  grafiekxLabel = xLabel;
  grafiekyLabel = yLabel;

  attrGezet = True;

  Refresh();
  return(Ok);
}

FoutCode EenDimensieGrafiek::zetTekenSektie(const UInt16 sektie)          
{
    wxASSERT_MSG(sektie*grafiekGrootte.x < dataGrootte,
		 wxT("Te hoog sektienummer opgegeven "));

    afbeeldingsStart = sektie*grafiekGrootte.x;

    Refresh();
    return(Ok);
}

FoutCode EenDimensieGrafiek::zetSchaalGrootte(const UInt16 sch)
{
  wxASSERT_MSG(sch != 0,wxT("(zetSchaalGrootte) opgegeven schaal = 0"));

  return(Ok);

}

FoutCode EenDimensieGrafiek::tekenassenstelsel(wxPaintDC *dc)
{

  UInt16 strepenTeller, streepLengte;

  wxCoord karX,karY;
  wxString veld,tmp;

  UInt16 coord;


  dc->SetPen( wxPen( assenstelselKleur, 2, wxSOLID));
  dc->SetBrush(assenstelselBlokKwast);

  /* teken de x -as en y-as */

  /* x-as */

  dc->DrawLine((wxCoord)(grafiekSchaalX*negatieveLimiet.x), 
	       0, 
	       (wxCoord)(grafiekSchaalX*(negatieveLimiet.x + grafiekGrootte.x)), 
	       0);

  /* y -as */
  dc->DrawLine(0,
	       (wxCoord)(grafiekSchaalY*(negatieveLimiet.y)), 
	       0,
	       (wxCoord)(grafiekSchaalY*(negatieveLimiet.y  + grafiekGrootte.y  )) );

  /* strepen langs de x as */
  for (strepenTeller=0;strepenTeller < grafiekGrootte.x ;strepenTeller ++)
    {
      coord = (wxCoord) (grafiekSchaalX*(negatieveLimiet.x + strepenTeller ));

      if (((strepenTeller+1) % 5) == 0)
      {
	  dc->SetPen( wxPen( assenstelselKleur, 4, wxSOLID));
	  streepLengte = 6;
	  veld.Printf("%d",1+afbeeldingsStart+strepenTeller );
	  GetTextExtent(veld,&karX, &karY);
	  dc->DrawText(veld, coord,-1*(karY+3));
      }
      else
      {
	  dc->SetPen( wxPen( assenstelselKleur, 2, wxSOLID));
	  streepLengte = 3;
      }

//      wxLogDebug(wxT("lijn (%d,%d) --> (%d,%d)"),coord, -1* streepLengte, coord, streepLengte);
      dc->DrawLine(coord, -1* streepLengte,
		   coord, streepLengte) ;
    }

  /* strepen langs de y-as */
  for (strepenTeller=0;strepenTeller <  grafiekGrootte.y;strepenTeller ++)
    {
      if (((strepenTeller+1) % 5) == 0)
	streepLengte = 4;
      else
	streepLengte = 2;

      coord =  (wxCoord) (grafiekSchaalY*( negatieveLimiet.y + strepenTeller  ));
      dc->DrawLine(-1*streepLengte, coord,
		   streepLengte, coord );
    }


  /* Geef Legenda  aan */

  dc->SetTextForeground( "GREY" );

  /* X - as */
  tmp = grafiekxLabel;
  veld = "+" + tmp;
  dc->DrawText(veld, 
	       (wxCoord)(grafiekSchaalX*(negatieveLimiet.x + grafiekGrootte.x) + 5),
	       0 );

  /* Y - as */
  tmp = grafiekyLabel;
  veld = "+" + tmp;
  GetTextExtent(veld,&karX, &karY);
  dc->DrawText(veld,  
	       0,
	       (wxCoord) (grafiekSchaalY*( negatieveLimiet.y + grafiekGrootte.y) + 5 + karY));

  veld = "-" + tmp;
  dc->DrawText(veld,  
	       0,
	       (wxCoord) (grafiekSchaalY*( negatieveLimiet.y + grafiekGrootte.y) -5 ));

  return(Ok);

}


void EenDimensieGrafiek::teken(wxPaintEvent &event)
{
  if (! (attrGezet && dataGeladen && assenstelselGezet) )
    event.Skip();
  else
    {
      wxPaintDC dc(this);

      wxCoord xOffSet,yCoord;
      Teller i;
      wxSize grootte;

      wxString xLabel, yLabel,waardeVeld;

      PrepareDC(dc);

      dc.Clear();

      /* bepaal de dimensies van de dc */
      grootte = dc.GetSize();

      grafiekSchaalX = 1.0*grootte.GetWidth()/grafiekGrootte.x;
      grafiekSchaalY = (1.0*schaal*grootte.GetHeight())/grafiekGrootte.y;

      /*     eenheidGrootte.x = grafiekGrootte.x/grootte.GetWidth();
      wxLogDebug(wxT("eenheidgrootte.x = %d ( %d / %d )"), eenheidGrootte.x,grafiekGrootte.x,grootte.GetWidth() );
      
      eenheidGrootte.y = grafiekGrootte.y/grootte.GetHeight();
      wxLogDebug(wxT("eenheidgrootte.y = %d ( %d / %d )"), eenheidGrootte.y,grafiekGrootte.y,grootte.GetHeight() );
      */

      /* zet het font */
      dc.SetFont( wxFont(10, 
			 wxMODERN, 
			 wxNORMAL,
			 wxNORMAL,
			 False,
			 "",
			 wxFONTENCODING_SYSTEM ));


      /* zet de oorsprong linksonder met opgeteld het nieuwe centrum */
      dc.SetDeviceOrigin((wxCoord)(grafiekSchaalX*grafiekBasis.x),
			 grootte.GetHeight()+ (wxCoord)(grafiekSchaalY*grafiekBasis.y));

      /* zet het dc assenstelsel zolas wij zelf tekenen */
      dc.SetAxisOrientation(True, True);

      /* teken het assenstelsel */
      tekenassenstelsel( &dc);

      dc.SetBrush( grafiekBlokKwast);

/*      wxLogDebug(wxT("(teken): data (totaal :%d) [%ld --> %ld] wordt getekend"),
		 dataGrootte,
		 (UInt32) afbeeldingsStart+1,
		 (UInt32) afbeeldingsStart+(Teller)grafiekGrootte.x );
*/
      /* teken de grafiek */

      /* offset zodat alles daar begint */
      xOffSet = (wxCoord) (grafiekSchaalX*((data+afbeeldingsStart)->x));

      for (i=afbeeldingsStart+1;i<afbeeldingsStart+(Teller)grafiekGrootte.x;i++)
	{
	  if (i==dataGrootte)
	  {
	      wxLogDebug(wxT("(teken) i == dataGrootte")); 
	      break;
	  }
	  else
	    {

	      /* data */
	      dc.SetPen( wxPen( grafiekKleur, 2, wxSOLID));

/*	      wxLogDebug(wxT("(teken):data = (%f,%f) --> (%f,%f)"),
			 (Float)(grafiekSchaalX*(data+i-1)->x), 
			 (Float) 0,
			 (Float)(grafiekSchaalX*( (data+i)->x - (data+i-1)->x)),
			 (Float)(grafiekSchaalY*(data+i-1)->y));

	      wxLogDebug(wxT("(teken):plot = (%ld,%ld) --> (%ld,%ld)"),
			 (UInt32) (grafiekSchaalX*(data+i-1)->x), 
			 (UInt32) 0,
			 (UInt32) (grafiekSchaalX*( (data+i)->x - (data+i-1)->x)),
			 (UInt32) (grafiekSchaalY*(data+i-1)->y));
*/
	      dc.DrawRectangle((wxCoord) (grafiekSchaalX*(data+i-1)->x) - xOffSet, 
			       (wxCoord) 0,
			       (wxCoord) (grafiekSchaalX*( (data+i)->x - (data+i-1)->x)),
			       (wxCoord) (grafiekSchaalY*(data+i-1)->y) ) ;


		  /* zet de pen voor de data labels */
	      if (tekenDataLabels)
	      {
		  dc.SetPen( wxPen("BROWN", 5, wxSOLID));

		  yCoord = (wxCoord)(grafiekSchaalY*( (data+afbeeldingsStart+getoondeDataXCoordinaat)->y ));

		  waardeVeld.Printf("%.2f",
				      (WiskundeFloatType)(1.0*schaal*(data+afbeeldingsStart+getoondeDataXCoordinaat)->y) );

		  dc.DrawText(waardeVeld, 
			      (wxCoord) (grafiekSchaalX*( (data+getoondeDataXCoordinaat)->x + (data+getoondeDataXCoordinaat+1)->x)/2 ) , 
			      yCoord );

	      }

	    }

	}
      dc.EndDrawing();
    }

}

void EenDimensieGrafiek::toonGrafiekWaarde(wxMouseEvent &event)
{
    wxCoord xCoord;
    wxPoint cursorCoordinaten(event.GetPosition());

    xCoord = (wxCoord) (1.0*cursorCoordinaten.x / grafiekSchaalX) ;

    if (xCoord != getoondeDataXCoordinaat)
    {
	getoondeDataXCoordinaat = xCoord;
	Refresh();
    }
}


FoutCode EenDimensieGrafiek::assenstelselZetKleur(wxColour kleur)
{
  assenstelselKleur = kleur;


  Refresh();
  return(Ok);
}



FoutCode EenDimensieGrafiek::grafiekZetKleur(wxColour kleur)
{

  grafiekKleur = kleur;
  grafiekBlokKwast.SetColour(kleur);

  Refresh();
  return(Ok);
}

FoutCode EenDimensieGrafiek::dataLabelSchakelaar(const Bool8 stand)
{

  tekenDataLabels = stand;

  Refresh();

  return(Ok);
}


/********************************* Staafgrafiek ****************************/

BEGIN_EVENT_TABLE( StaafGrafiek, wxPanel)

    EVT_PAINT(StaafGrafiek::teken)

END_EVENT_TABLE();


StaafGrafiek::StaafGrafiek(wxWindow* parent, 
				       int id,
				       const wxPoint &pos, 
				       const wxSize &size, 
				       long style):
    wxPanel(parent, id, pos, size, wxTAB_TRAVERSAL)
{

    /* data labels */
    tekenDataLabels = True;

    attrGezet = assenstelselGezet= False;


}


StaafGrafiek::~StaafGrafiek()
{

}


FoutCode StaafGrafiek::zetDataPunt(const WiskundeFloatType waarde)
{


  data = waarde;

  Refresh();

  return(Ok);


}


FoutCode StaafGrafiek::zetAssenstelsel(wxSize negatiefBereik,  
				       wxSize positiefBereik,
				       const wxSize iAfstand)       /* aantal ticks tussen labels */


{
  grafiekGrootte.x = positiefBereik.GetWidth() - negatiefBereik.GetWidth();

  grafiekGrootte.y = positiefBereik.GetHeight() - negatiefBereik.GetHeight();
  negatieveLimiet =  negatiefBereik;

  grafiekBasis.x = negatiefBereik.GetWidth();
  grafiekBasis.y = negatiefBereik.GetHeight();
 
  /* Bepaal de breedte van een interval*/
  intervalAfstand =  iAfstand;

  assenstelselGezet = True;

  Refresh();

  return(Ok);
}

FoutCode StaafGrafiek::zetAttributen(wxColour asKleur,
				     wxBrush asKwast,
				     wxColour grafKleur,
				     wxBrush grafKwast,
				     wxString xLabel,
				     wxString yLabel )

{

  assenstelselKleur = asKleur;
  assenstelselBlokKwast = asKwast;

  grafiekKleur = grafKleur;
  grafiekBlokKwast = grafKwast;

  grafiekxLabel = xLabel;
  grafiekyLabel = yLabel;

  attrGezet = True;

  Refresh();
  return(Ok);
}


FoutCode StaafGrafiek::zetSchaalGrootte(const UInt8 sch)
{
  wxASSERT_MSG(sch != 0,wxT("(zetSchaalGrootte) opgegeven schaal = 0"));

  schaal = sch;



  return(Ok);
}


FoutCode StaafGrafiek::tekenassenstelsel(wxPaintDC *dc)
{

  UInt16 strepenTeller, streepLengte;

  wxCoord karX,karY;
  wxString veld,tmp;

  UInt16 coord;


  dc->SetPen( wxPen( assenstelselKleur, 2, wxSOLID));
  dc->SetBrush(assenstelselBlokKwast);

  /* teken de x -as en y-as */

  /* x-as */

  dc->DrawLine((wxCoord)(grafiekSchaalX*negatieveLimiet.x), 
	       0, 
	       (wxCoord)(grafiekSchaalX*(negatieveLimiet.x + grafiekGrootte.x)), 
	       0);

  /* y -as */
  dc->DrawLine(0,
	       (wxCoord)(grafiekSchaalY*(negatieveLimiet.y)), 
	       0,
	       (wxCoord)(grafiekSchaalY*(negatieveLimiet.y  + grafiekGrootte.y  )) );

  /* strepen langs de x as */
  for (strepenTeller=0;strepenTeller < grafiekGrootte.x ;strepenTeller ++)
    {
      coord = (wxCoord) (grafiekSchaalX*(negatieveLimiet.x + strepenTeller ));

      if (((strepenTeller+1) % 5) == 0)
      {
	  dc->SetPen( wxPen( assenstelselKleur, 4, wxSOLID));
	  streepLengte = 6;
	  veld.Printf("%d",1+strepenTeller );
	  GetTextExtent(veld,&karX, &karY);
	  dc->DrawText(veld, coord,-1*(karY+3));
      }
      else
      {
	  dc->SetPen( wxPen( assenstelselKleur, 2, wxSOLID));
	  streepLengte = 3;
      }

//      wxLogDebug(wxT("lijn (%d,%d) --> (%d,%d)"),coord, -1* streepLengte, coord, streepLengte);
      dc->DrawLine(coord, -1* streepLengte,
		   coord, streepLengte) ;
    }

  /* strepen langs de y-as */
  for (strepenTeller=0;strepenTeller <  grafiekGrootte.y;strepenTeller ++)
    {
      if (((strepenTeller+1) % 5) == 0)
	streepLengte = 4;
      else
	streepLengte = 2;

      coord =  (wxCoord) (grafiekSchaalY*( negatieveLimiet.y + strepenTeller  ));
      dc->DrawLine(-1*streepLengte, coord,
		   streepLengte, coord );
    }


  /* Geef Legenda  aan */

  dc->SetTextForeground( "GREY" );

  /* X - as */
  tmp = grafiekxLabel;
  veld = "+" + tmp;
  dc->DrawText(veld, 
	       (wxCoord)(grafiekSchaalX*(negatieveLimiet.x + grafiekGrootte.x) + 5),
	       0 );

  /* Y - as */
  tmp = grafiekyLabel;
  veld = "+" + tmp;
  GetTextExtent(veld,&karX, &karY);
  dc->DrawText(veld,  
	       0,
	       (wxCoord) (grafiekSchaalY*( negatieveLimiet.y + grafiekGrootte.y) + 5 + karY));

  veld = "-" + tmp;
  dc->DrawText(veld,  
	       0,
	       (wxCoord) (grafiekSchaalY*( negatieveLimiet.y + grafiekGrootte.y) -5 ));

  return(Ok);

}


void StaafGrafiek::teken(wxPaintEvent &event)
{

  if (! (attrGezet && assenstelselGezet) )
    event.Skip();
  else
    {
      wxPaintDC dc(this);

      wxSize grootte;

      wxString xLabel, yLabel,waardeVeld;

      PrepareDC(dc);

      dc.Clear();

      /* bepaal de dimensies van de dc */
      grootte = dc.GetSize();

      grafiekSchaalX = 1.0*schaal*grootte.GetWidth()/grafiekGrootte.x;
      grafiekSchaalY = (1.0*schaal*grootte.GetHeight())/grafiekGrootte.y;

      /*     eenheidGrootte.x = grafiekGrootte.x/grootte.GetWidth();
	     wxLogDebug(wxT("eenheidgrootte.x = %d ( %d / %d )"), eenheidGrootte.x,grafiekGrootte.x,grootte.GetWidth() );
      
	     eenheidGrootte.y = grafiekGrootte.y/grootte.GetHeight();
	     wxLogDebug(wxT("eenheidgrootte.y = %d ( %d / %d )"), eenheidGrootte.y,grafiekGrootte.y,grootte.GetHeight() );
      */

      /* zet het font */
      dc.SetFont( wxFont(10, 
			 wxMODERN, 
			 wxNORMAL,
			 wxNORMAL,
			 False,
			 "",
			 wxFONTENCODING_SYSTEM ));


      /* zet de oorsprong linksonder met opgeteld het nieuwe centrum */
      dc.SetDeviceOrigin((wxCoord)(grafiekSchaalX*grafiekBasis.x),
			 grootte.GetHeight()+ (wxCoord)(grafiekSchaalY*grafiekBasis.y));

      /* zet het dc assenstelsel zolas wij zelf tekenen */
      dc.SetAxisOrientation(True, True);

      /* teken het assenstelsel */
      tekenassenstelsel( &dc);

      dc.SetBrush( grafiekBlokKwast);

      /*      wxLogDebug(wxT("(teken): data (totaal :%d) [%ld --> %ld] wordt getekend"),
	      dataGrootte,
	      (UInt32) afbeeldingsStart+1,
	      (UInt32) afbeeldingsStart+(Teller)grafiekGrootte.x );
      */
      /* teken de grafiek */


      /* data */
      dc.SetPen( wxPen( grafiekKleur, 2, wxSOLID));

      /*	      wxLogDebug(wxT("(teken):data = (%f,%f) --> (%f,%f)"),
		      (Float)(grafiekSchaalX*(data+i-1)->x), 
		      (Float) 0,
		      (Float)(grafiekSchaalX*( (data+i)->x - (data+i-1)->x)),
		      (Float)(grafiekSchaalY*(data+i-1)->y));

		      wxLogDebug(wxT("(teken):plot = (%ld,%ld) --> (%ld,%ld)"),
		      (UInt32) (grafiekSchaalX*(data+i-1)->x), 
		      (UInt32) 0,
		      (UInt32) (grafiekSchaalX*( (data+i)->x - (data+i-1)->x)),
		      (UInt32) (grafiekSchaalY*(data+i-1)->y));
      */
      dc.DrawRectangle( 0,
		        0,
			(wxCoord) (grafiekSchaalX*data),
			(wxCoord) 20 ) ;
      

      /* zet de pen voor de data labels */
      if (tekenDataLabels)
	{
	  dc.SetPen( wxPen("BROWN", 5, wxSOLID));
	  waardeVeld.Printf("%2.2f",data );

	  dc.DrawText(waardeVeld, 
		      (wxCoord) (grafiekSchaalX*data),
		      10 );



	}

      dc.EndDrawing();
    }


}




FoutCode StaafGrafiek::assenstelselZetKleur(wxColour kleur)
{
  assenstelselKleur = kleur;


  Refresh();
  return(Ok);
}



FoutCode StaafGrafiek::grafiekZetKleur(wxColour kleur)
{

  grafiekKleur = kleur;
  grafiekBlokKwast.SetColour(kleur);

  Refresh();
  return(Ok);
}

FoutCode StaafGrafiek::dataLabelSchakelaar(const Bool8 stand)
{

  tekenDataLabels = stand;

  Refresh();

  return(Ok);
}
