/************************************************************************
grafische funkties API. Hier staat wxWidget code om grafieken in
assenstelsels te tekenen.

(c) 2004-2005 Kiwanda Embedded Systemen
$Id: grafiek.h,v 1.2 2006/04/07 22:11:01 oldenburgh Exp $
************************************************************************/

#ifndef KIWANDA_GRAFIEK_H
#define KIWANDA_GRAFIEK_H

/* GUI includes */
#include <wx/panel.h>
#include <wx/brush.h>
#include <wx/dcclient.h>

#define KIWANDA_INTEL386

#include "algdef.h"


class EenDimensieGrafiek : public wxPanel
{

 public:
    EenDimensieGrafiek(wxWindow* parent, 
                       int id, 
                       const wxPoint &pos=wxDefaultPosition, 
                       const wxSize &size=wxDefaultSize, 
                       long style=0);

    ~EenDimensieGrafiek();

     /* initialiseer  de dataruimte. Dit moet gebeuren voor enige andere funktieaanroep */
    FoutCode vraagDataRuimte(const UInt16 );          /* aantal */
    /* Geef de dataruimte weer terug */       
    FoutCode geefDataRuimteOp();     

    FoutCode zetAssenstelsel(wxSize ,       /* negatief bereik */  
                             wxSize ,       /* positief bereik */
                             const wxSize ,       /* aantal ticks tussen labels */
                             const Bool8,         /* auto Y schaal instellen */
                             UInt16 *);       /* return: aantal
                                                 secties in de
                                                 dataset die
                                                 afgebeeld kunnen
                                                 worden in het
                                                 assenstelsel */

    FoutCode zetAttributen(wxColour,          /* assenstelsel */
                           wxBrush,           /* assenstelsel kwast */       
                           wxColour,          /* grafiek      */ 
                           wxBrush,           /* grafiek kwast */  
                           wxString ,         /* label voor x as */
                           wxString  );       /* label voor y as */


    FoutCode zetAlleData(const wxRealPoint *);        /* zet alle data ineens */

    FoutCode zetDataPunt(UInt16 nummer,
                         const wxRealPoint );                   /* nummer van het punt */

    FoutCode voegDataPuntToe(const wxRealPoint );     /* voegt een punt toe op de volgende plek */

    FoutCode zetTekenSektie(const UInt16 );           /* geef de sektie aan die getekend moet worden */

    FoutCode zetSchaalGrootte(const UInt16);           /* Y-as label waarde wordt vermenigvuldigd met deze waarde */

    FoutCode dataLabelSchakelaar(const Bool8 );       /* zet labels aan of uit */

    FoutCode assenstelselZetKleur(wxColour );         /* zet kleur van het assenstelsel */
    FoutCode grafiekZetKleur(wxColour );              /* zet de kleur van de grafiek */ 

 private:

    /* data geladen */
    Bool8 dataGeladen;
    Bool8 attrGezet,assenstelselGezet;

    /* dataset */
    UInt16 dataGrootte;           /* hoeveelheid af te beelden data */
    UInt16 toevoegPunt;           /* punt in array waar we data toevoegen */
    wxRealPoint *data;            /* bevat de af te beelden dat */
    UInt16 afbeeldingsStart;      /* welke set (indien assenstelsel < dataset ) beelden we af */
    UInt16 schaal;                 /* schaal van de y-as van de te tekenen data */

    /* Grafiekmogelijkheden */
    Bool8 tekenDataLabels;

    /* Toon label bij data waar de cursor het dichtst bij staat */
    wxCoord getoondeDataXCoordinaat; /* huidige getoonde waarde */

    /* teken de grafieken attributen */
    FoutCode tekenassenstelsel(wxPaintDC *);



    wxSize assenstelselNegGrootte;
    wxSize assenstelselPosGrootte;
    wxSize assenstelselTicks;

    /* Attributen */
    wxColour assenstelselKleur;
    wxBrush assenstelselBlokKwast;

    wxColour grafiekKleur;
    wxBrush grafiekBlokKwast;


    wxSize grafiekStrepen;
    wxString grafiekxLabel;
    wxString grafiekyLabel;


    WiskundeFloatType maximum;

    /* mapping van grafiek naar DC */
    WiskundeFloatType grafiekSchaalX, grafiekSchaalY;   /* verhouding DC grootte / grafiekgrootte */
    //   wxSize eenheidGrootte;                              /* grootte (in pixels) van (0,0) - (1,1)  */

    /* referentie variabelen */
    //Assenstelsels assenStelsel;

    wxSize negatieveLimiet;         /* meest negatieve coordinaat in assenstelsel */
    wxSize grafiekGrootte;          /* grootte van het assenstelsel */
    wxPoint grafiekBasis;           /* lokatie van de oorsprong */
    wxSize intervalAfstand;         /* afstand tussen tekstlabels */

 protected:

    void teken(wxPaintEvent  &);
    void toonGrafiekWaarde(wxMouseEvent &);

    DECLARE_EVENT_TABLE();


};

class StaafGrafiek : public wxPanel
{

 public:
  StaafGrafiek(wxWindow* parent, 
                       int id, 
                       const wxPoint &pos=wxDefaultPosition, 
                       const wxSize &size=wxDefaultSize, 
                       long style=0);

    ~StaafGrafiek();

     /* initialiseer  de dataruimte. Dit moet gebeuren voor enige andere funktieaanroep */

    FoutCode zetAssenstelsel(wxSize ,       /* negatief bereik */  
                             wxSize ,       /* positief bereik */
                             const wxSize );       /* aantal ticks tussen labels */

    FoutCode zetAttributen(wxColour,          /* assenstelsel */
                           wxBrush,           /* assenstelsel kwast */       
                           wxColour,          /* grafiek      */ 
                           wxBrush,           /* grafiek kwast */  
                           wxString ,         /* label voor x as */
                           wxString  );       /* label voor y as */


    FoutCode zetDataPunt(const WiskundeFloatType );                   /* nummer van het punt */

    FoutCode zetSchaalGrootte(const UInt8);           /* Y-as wordt gedeeld door deze grootte */

    FoutCode dataLabelSchakelaar(const Bool8 );       /* zet labels aan of uit */

    FoutCode assenstelselZetKleur(wxColour );         /* zet kleur van het assenstelsel */
    FoutCode grafiekZetKleur(wxColour );              /* zet de kleur van de grafiek */ 

 private:

    /* data geladen */
    Bool8 attrGezet,assenstelselGezet;

    /* dataset */
    WiskundeFloatType data;
    UInt8 schaal;                 /* schaal van de y-as van de te tekenen data */

    /* Grafiekmogelijkheden */
    Bool8 tekenDataLabels;

    /* teken de grafieken attributen */
    FoutCode tekenassenstelsel(wxPaintDC *);

    wxSize assenstelselNegGrootte;
    wxSize assenstelselPosGrootte;
    wxSize assenstelselTicks;

    /* Attributen */
    wxColour assenstelselKleur;
    wxBrush assenstelselBlokKwast;

    wxColour grafiekKleur;
    wxBrush grafiekBlokKwast;


    wxSize grafiekStrepen;
    wxString grafiekxLabel;
    wxString grafiekyLabel;

    WiskundeFloatType maximum;

    /* mapping van grafiek naar DC */
    WiskundeFloatType grafiekSchaalX, grafiekSchaalY;   /* verhouding DC grootte / grafiekgrootte */

    /* referentie variabelen */
    //Assenstelsels assenStelsel;

    wxSize negatieveLimiet;         /* meest negatieve coordinaat in assenstelsel */
    wxSize grafiekGrootte;          /* grootte van het assenstelsel */
    wxPoint grafiekBasis;           /* lokatie van de oorsprong */
    wxSize intervalAfstand;         /* afstand tussen tekstlabels */

 protected:

    void teken(wxPaintEvent  &);

    DECLARE_EVENT_TABLE();


};






#endif   /* #ifndef */
